@@ -1,18 +1,19 @@
 /*
  * Copyright (C) 2002 Red Hat, Inc.
  *
- * This library is free software: you can redistribute it and/or modify
- * it under the terms of the GNU Lesser General Public License as published
- * by the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU Lesser General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public License
- * along with this library.  If not, see <https://www.gnu.org/licenses/>.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 #include <config.h>
@@ -22,7 +23,9 @@
 #ifdef HAVE_SYS_SELECT_H
 #include <sys/select.h>
 #endif
-#include <termios.h>
+#ifdef HAVE_SYS_TERMIOS_H
+#include <sys/termios.h>
+#endif
 #include <sys/time.h>
 #include <unistd.h>
 #include <fcntl.h>
@@ -30,6 +33,9 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#ifdef HAVE_TERMIOS_H
+#include <termios.h>
+#endif
 #include <unistd.h>
 #include <glib.h>
 
@@ -37,11 +43,17 @@
 #define MODE_APPLICATION_KEYPAD		ESC "="
 #define MODE_NORMAL_KEYPAD		ESC ">"
 #define MODE_APPLICATION_CURSOR_KEYS	1
+#define MODE_SUN_FUNCTION_KEYS		1051
+#define MODE_HP_FUNCTION_KEYS		1052
+#define MODE_XTERM_FUNCTION_KEYS	1060
+#define MODE_VT220_FUNCTION_KEYS	1061
 #define MODE_ALTERNATE_SCREEN		1047
 
 enum {
 	normal = 0, application = 1
 } keypad_mode = normal, cursor_mode = normal;
+gboolean sun_fkeys = FALSE, hp_fkeys = FALSE,
+	 xterm_fkeys = FALSE, vt220_fkeys = FALSE;
 struct termios original;
 
 /* Output the DEC private mode set sequence. */
@@ -84,6 +96,30 @@ print_help(void)
 	} else {
 		g_print("NORMAL\r\n");
 	}
+	g_print(ESC "[K" "C - SUN    ");
+	if (sun_fkeys) {
+		g_print("TRUE\r\n");
+	} else {
+		g_print("FALSE\r\n");
+	}
+	g_print(ESC "[K" "D - HP     ");
+	if (hp_fkeys) {
+		g_print("TRUE\r\n");
+	} else {
+		g_print("FALSE\r\n");
+	}
+	g_print(ESC "[K" "E - XTERM  ");
+	if (xterm_fkeys) {
+		g_print("TRUE\r\n");
+	} else {
+		g_print("FALSE\r\n");
+	}
+	g_print(ESC "[K" "F - VT220  ");
+	if (vt220_fkeys) {
+		g_print("TRUE\r\n");
+	} else {
+		g_print("FALSE\r\n");
+	}
 	g_print(ESC "[K" "R - RESET\r\n");
 	g_print(ESC "[K" "Q - QUIT\r\n");
 }
@@ -101,8 +137,8 @@ reset_scrolling_region(void)
 static void
 set_scrolling_region(void)
 {
-	g_print(ESC "[6;24r");
-	g_print(ESC "[5;1H");
+	g_print(ESC "[10;24r");
+	g_print(ESC "[9;1H");
 }
 
 /* Save the current location of the cursor in the terminal's memory. */
@@ -125,6 +161,10 @@ reset(void)
 {
 	g_print(MODE_NORMAL_KEYPAD);
 	decset(MODE_APPLICATION_CURSOR_KEYS, FALSE);
+	decset(MODE_SUN_FUNCTION_KEYS, FALSE);
+	decset(MODE_HP_FUNCTION_KEYS, FALSE);
+	decset(MODE_XTERM_FUNCTION_KEYS, FALSE);
+	decset(MODE_VT220_FUNCTION_KEYS, FALSE);
 	reset_scrolling_region();
 	restore_cursor();
 }
@@ -161,7 +201,17 @@ main(int argc, char **argv)
 	}
 	original = tcattr;
 	signal(SIGINT, sigint_handler);
+	/* Here we approximate what cfmakeraw() would do, for the benefit
+	 * of systems which don't actually provide the function. */
+	tcattr.c_iflag &= ~(IGNBRK | BRKINT | PARMRK | ISTRIP |
+			    INLCR | IGNCR | ICRNL | IXON);
+	tcattr.c_oflag &= ~(OPOST);
+	tcattr.c_lflag &= ~(ECHO | ECHONL | ICANON | ISIG | IEXTEN);
+	tcattr.c_cflag &= ~(CSIZE | PARENB);
+	tcattr.c_cflag |= CS8;
+#ifdef HAVE_CFMAKERAW
 	cfmakeraw(&tcattr);
+#endif
 	if (tcsetattr(STDIN_FILENO, TCSANOW, &tcattr) != 0) {
 		perror("tcsetattr");
 		return 1;
@@ -200,9 +250,30 @@ main(int argc, char **argv)
 			decset(MODE_APPLICATION_CURSOR_KEYS,
 			       cursor_mode == application);
 			break;
+		case 'C':
+		case 'c':
+			sun_fkeys = !sun_fkeys;
+			decset(MODE_SUN_FUNCTION_KEYS, sun_fkeys);
+			break;
+		case 'D':
+		case 'd':
+			hp_fkeys = !hp_fkeys;
+			decset(MODE_HP_FUNCTION_KEYS, hp_fkeys);
+			break;
+		case 'E':
+		case 'e':
+			xterm_fkeys = !xterm_fkeys;
+			decset(MODE_XTERM_FUNCTION_KEYS, xterm_fkeys);
+			break;
+		case 'F':
+		case 'f':
+			vt220_fkeys = !vt220_fkeys;
+			decset(MODE_VT220_FUNCTION_KEYS, vt220_fkeys);
+			break;
 		case 'R':
 		case 'r':
 			keypad_mode = cursor_mode = normal;
+			sun_fkeys = hp_fkeys = xterm_fkeys = vt220_fkeys = FALSE;
 			reset();
 			break;
 		case 'Q':
