@@ -25,6 +25,7 @@
 #include "caps.h"
 #include "matcher.h"
 #include "table.h"
+#include "trie.h"
 
 struct _vte_matcher {
 	_vte_matcher_match_func match; /* shortcut to the most common op */
@@ -32,10 +33,11 @@ struct _vte_matcher {
 	GValueArray *free_params;
 };
 
-static GMutex _vte_matcher_mutex;
-static struct _vte_matcher *_vte_matcher_singleton = NULL;
-static int _vte_matcher_ref_count = 0;
-
+static GStaticMutex _vte_matcher_mutex = G_STATIC_MUTEX_INIT;
+static GCache *_vte_matcher_cache = NULL;
+static struct _vte_matcher_impl dummy_vte_matcher_trie = {
+	&_vte_matcher_trie
+};
 static struct _vte_matcher_impl dummy_vte_matcher_table = {
 	&_vte_matcher_table
 };
@@ -44,52 +46,113 @@ static struct _vte_matcher_impl dummy_vte_matcher_table = {
 static void
 _vte_matcher_add(const struct _vte_matcher *matcher,
 		 const char *pattern, gssize length,
-		 const char *result)
+		 const char *result, GQuark quark)
 {
-	matcher->impl->klass->add(matcher->impl, pattern, length, result);
+	matcher->impl->klass->add(matcher->impl, pattern, length, result, quark);
 }
 
 /* Loads all sequences into matcher */
 static void
-_vte_matcher_init(struct _vte_matcher *matcher)
+_vte_matcher_init(struct _vte_matcher *matcher, const char *emulation,
+		  struct _vte_termcap *termcap)
 {
 	const char *code, *value;
+	gboolean found_cr = FALSE, found_lf = FALSE;
+	gssize stripped_length;
+	char *stripped;
+	int i;
 
 	_vte_debug_print(VTE_DEBUG_LIFECYCLE, "_vte_matcher_init()\n");
 
-        code = _vte_xterm_capability_strings;
-        do {
-                value = strchr(code, '\0') + 1;
-                _vte_matcher_add(matcher, code, strlen(code), value);
-                code = strchr(value, '\0') + 1;
-        } while (*code);
+	if (termcap != NULL) {
+		/* Load the known capability strings from the termcap
+		 * structure into the table for recognition. */
+		for (i = 0;
+				_vte_terminal_capability_strings[i].capability[0];
+				i++) {
+			if (_vte_terminal_capability_strings[i].key) {
+				continue;
+			}
+			code = _vte_terminal_capability_strings[i].capability;
+			stripped = _vte_termcap_find_string_length(termcap,
+					emulation,
+					code,
+					&stripped_length);
+			if (stripped[0] != '\0') {
+				_vte_matcher_add(matcher,
+						stripped, stripped_length,
+						code, 0);
+				if (stripped[0] == '\r') {
+					found_cr = TRUE;
+				} else
+					if (stripped[0] == '\n') {
+						if (strcmp(code, "sf") == 0 ||
+								strcmp(code, "do") == 0) {
+							found_lf = TRUE;
+						}
+					}
+			}
+			g_free(stripped);
+		}
+	}
 
-	_VTE_DEBUG_IF(VTE_DEBUG_MATCHER) {
-		g_printerr("Matcher contents:\n");
+	/* Add emulator-specific sequences. */
+	if (strstr(emulation, "xterm") || strstr(emulation, "dtterm")) {
+		/* Add all of the xterm-specific stuff. */
+		for (i = 0;
+		     _vte_xterm_capability_strings[i].value != NULL;
+		     i++) {
+			code = _vte_xterm_capability_strings[i].code;
+			value = _vte_xterm_capability_strings[i].value;
+			_vte_matcher_add(matcher, code, strlen (code),
+					 value, 0);
+		}
+	}
+
+	/* Always define cr and lf. */
+	if (!found_cr) {
+		_vte_matcher_add(matcher, "\r", 1, "cr", 0);
+	}
+	if (!found_lf) {
+		_vte_matcher_add(matcher, "\n", 1, "sf", 0);
+	}
+
+	_VTE_DEBUG_IF(VTE_DEBUG_TRIE) {
+		g_printerr("Trie contents:\n");
 		_vte_matcher_print(matcher);
 		g_printerr("\n");
 	}
 }
 
 /* Allocates new matcher structure. */
-static struct _vte_matcher *
-_vte_matcher_create(void)
+static gpointer
+_vte_matcher_create(gpointer key)
 {
+	char *emulation = key;
 	struct _vte_matcher *ret = NULL;
 
 	_vte_debug_print(VTE_DEBUG_LIFECYCLE, "_vte_matcher_create()\n");
 	ret = g_slice_new(struct _vte_matcher);
-        ret->impl = &dummy_vte_matcher_table;
+	ret->impl = &dummy_vte_matcher_trie;
 	ret->match = NULL;
 	ret->free_params = NULL;
 
+	if (strcmp(emulation, "xterm") == 0) {
+		ret->impl = &dummy_vte_matcher_table;
+	} else
+	if (strcmp(emulation, "dtterm") == 0) {
+		ret->impl = &dummy_vte_matcher_table;
+	}
+
 	return ret;
 }
 
 /* Noone uses this matcher, free it. */
 static void
-_vte_matcher_destroy(struct _vte_matcher *matcher)
+_vte_matcher_destroy(gpointer value)
 {
+	struct _vte_matcher *matcher = value;
+
 	_vte_debug_print(VTE_DEBUG_LIFECYCLE, "_vte_matcher_destroy()\n");
 	if (matcher->free_params != NULL) {
 		g_value_array_free (matcher->free_params);
@@ -101,39 +164,42 @@ _vte_matcher_destroy(struct _vte_matcher *matcher)
 
 /* Create and init matcher. */
 struct _vte_matcher *
-_vte_matcher_new(void)
+_vte_matcher_new(const char *emulation, struct _vte_termcap *termcap)
 {
 	struct _vte_matcher *ret = NULL;
+	g_static_mutex_lock(&_vte_matcher_mutex);
 
-        g_mutex_lock(&_vte_matcher_mutex);
-
-        if (_vte_matcher_ref_count++ == 0) {
-                g_assert(_vte_matcher_singleton == NULL);
-                ret = _vte_matcher_create();
-
-                if (ret->match == NULL) {
-                        ret->impl = ret->impl->klass->create();
-                        ret->match = ret->impl->klass->match;
-                        _vte_matcher_init(ret);
-                }
-                _vte_matcher_singleton = ret;
+	if (emulation == NULL) {
+		emulation = "";
 	}
 
-        g_mutex_unlock(&_vte_matcher_mutex);
-        return _vte_matcher_singleton;
+	if (_vte_matcher_cache == NULL) {
+		_vte_matcher_cache = g_cache_new(_vte_matcher_create,
+				_vte_matcher_destroy,
+				(GCacheDupFunc) g_strdup, g_free,
+				g_str_hash, g_direct_hash, g_str_equal);
+	}
+
+	ret = g_cache_insert(_vte_matcher_cache, (gpointer) emulation);
+
+	if (ret->match == NULL) {
+		ret->impl = ret->impl->klass->create();
+		ret->match = ret->impl->klass->match;
+		_vte_matcher_init(ret, emulation, termcap);
+	}
+
+	g_static_mutex_unlock(&_vte_matcher_mutex);
+	return ret;
 }
 
 /* Free a matcher. */
 void
 _vte_matcher_free(struct _vte_matcher *matcher)
 {
-        g_assert(_vte_matcher_singleton != NULL);
-        g_mutex_lock(&_vte_matcher_mutex);
-        if (--_vte_matcher_ref_count == 0) {
-                _vte_matcher_destroy(matcher);
-                _vte_matcher_singleton = NULL;
-        }
-        g_mutex_unlock(&_vte_matcher_mutex);
+	g_assert(_vte_matcher_cache != NULL);
+	g_static_mutex_lock(&_vte_matcher_mutex);
+	g_cache_remove(_vte_matcher_cache, matcher);
+	g_static_mutex_unlock(&_vte_matcher_mutex);
 }
 
 /* Check if a string matches a sequence the matcher knows about. */
@@ -141,14 +207,14 @@ const char *
 _vte_matcher_match(struct _vte_matcher *matcher,
 		   const gunichar *pattern, gssize length,
 		   const char **res, const gunichar **consumed,
-		   GValueArray **array)
+		   GQuark *quark, GValueArray **array)
 {
 	if (G_UNLIKELY (array != NULL && matcher->free_params != NULL)) {
 		*array = matcher->free_params;
 		matcher->free_params = NULL;
 	}
 	return matcher->match(matcher->impl, pattern, length,
-					res, consumed, array);
+					res, consumed, quark, array);
 }
 
 /* Dump out the contents of a matcher, mainly for debugging. */
@@ -179,3 +245,4 @@ _vte_matcher_free_params_array(struct _vte_matcher *matcher,
 		params->n_values = 0;
 	}
 }
+
