@@ -40,9 +40,11 @@
 
 struct _vte_table {
 	struct _vte_matcher_impl impl;
+	GQuark resultq;
 	const char *result;
 	unsigned char *original;
 	gssize original_length;
+	int increment;
 	struct _vte_table *table_string;
 	struct _vte_table *table_number;
 	struct _vte_table *table_number_list;
@@ -187,11 +189,12 @@ _vte_table_free(struct _vte_table *table)
 	g_slice_free(struct _vte_table, table);
 }
 
+/* Add a string to the tree with the given increment value. */
 static void
 _vte_table_addi(struct _vte_table *table,
 		const unsigned char *original, gssize original_length,
 		const char *pattern, gssize length,
-		const char *result)
+		const char *result, GQuark quark, int inc)
 {
 	int i;
 	guint8 check;
@@ -211,19 +214,31 @@ _vte_table_addi(struct _vte_table *table,
 					  "`%s' and `%s' are indistinguishable.\n",
 					  table->result, result);
 
-		table->result = g_intern_string(result);
+		table->resultq = g_quark_from_string(result);
+		table->result = g_quark_to_string(table->resultq);
 		if (table->original != NULL) {
 			g_free(table->original);
 		}
 		table->original = g_memdup(original, original_length);
 		table->original_length = original_length;
+		table->increment = inc;
 		return;
 	}
 
 	/* All of the interesting arguments begin with '%'. */
 	if (pattern[0] == '%') {
+		/* Handle an increment. */
+		if (pattern[1] == 'i') {
+			_vte_table_addi(table, original, original_length,
+					pattern + 2, length - 2,
+					result, quark, inc + 1);
+			return;
+		}
+
 		/* Handle numeric parameters. */
-		if (pattern[1] == 'd') {
+		if ((pattern[1] == 'd') ||
+		    (pattern[1] == '2') ||
+		    (pattern[1] == '3')) {
 			/* Create a new subtable. */
 			if (table->table_number == NULL) {
 				subtable = _vte_table_new();
@@ -234,20 +249,23 @@ _vte_table_addi(struct _vte_table *table,
 			/* Add the rest of the string to the subtable. */
 			_vte_table_addi(subtable, original, original_length,
 					pattern + 2, length - 2,
-					result);
+					result, quark, inc);
 			return;
 		}
 
 		/* Handle variable-length parameters. */
-		if (pattern[1] == 'm') {
+		if ((pattern[1] == 'm') ||
+		    (pattern[1] == 'M')) {
 			/* Build the "new" original using the initial portion
 			 * of the original string and what's left after this
 			 * specifier. */
-			{
+			if (pattern[1] == 'm') {
 				int initial;
 				GByteArray *b;
 
 				initial = original_length - length;
+				/* 0 args; we use 'M' to signal that zero is
+				 * not allowed.  */
 				b = g_byte_array_new();
 				g_byte_array_set_size(b, 0);
 				g_byte_array_append(b, original, initial);
@@ -255,7 +273,7 @@ _vte_table_addi(struct _vte_table *table,
 				_vte_table_addi(table, b->data, b->len,
 						(const char *)b->data + initial,
 						b->len - initial,
-						result);
+						result, quark, inc);
 				g_byte_array_free(b, TRUE);
 			}
 			/* Create a new subtable. */
@@ -268,7 +286,7 @@ _vte_table_addi(struct _vte_table *table,
 			/* Add the rest of the string to the subtable. */
 			_vte_table_addi(subtable, original, original_length,
 					pattern + 2, length - 2,
-					result);
+					result, quark, inc);
 			return;
 		}
 
@@ -286,7 +304,7 @@ _vte_table_addi(struct _vte_table *table,
 			/* Add the rest of the string to the subtable. */
 			_vte_table_addi(subtable, original, original_length,
 					pattern + 2, length - 2,
-					result);
+					result, quark, inc);
 			return;
 		}
 
@@ -307,7 +325,7 @@ _vte_table_addi(struct _vte_table *table,
 			/* Add the rest of the string to the subtable. */
 			_vte_table_addi(subtable, original, original_length,
 					pattern + 2, length - 2,
-					result);
+					result, quark, inc);
 			return;
 		}
 
@@ -334,7 +352,7 @@ _vte_table_addi(struct _vte_table *table,
 				_vte_table_addi(subtable,
 						original, original_length,
 						pattern + 3, length - 3,
-						result);
+						result, quark, inc);
 			}
 			/* Also add a subtable for higher characters. */
 			if (table->table == NULL) {
@@ -351,7 +369,7 @@ _vte_table_addi(struct _vte_table *table,
 			/* Add the rest of the string to the subtable. */
 			_vte_table_addi(subtable, original, original_length,
 					pattern + 3, length - 3,
-					result);
+					result, quark, inc);
 			return;
 		}
 	}
@@ -374,26 +392,26 @@ _vte_table_addi(struct _vte_table *table,
 	/* Add the rest of the string to the subtable. */
 	_vte_table_addi(subtable, original, original_length,
 			pattern + 1, length - 1,
-			result);
+			result, quark, inc);
 }
 
 /* Add a string to the matching tree. */
 void
 _vte_table_add(struct _vte_table *table,
 	       const char *pattern, gssize length,
-	       const char *result)
+	       const char *result, GQuark quark)
 {
 	_vte_table_addi(table,
 			(const unsigned char *) pattern, length,
 			pattern, length,
-			result);
+			result, quark, 0);
 }
 
 /* Match a string in a subtree. */
 static const char *
 _vte_table_matchi(struct _vte_table *table,
 		  const gunichar *candidate, gssize length,
-		  const char **res, const gunichar **consumed,
+		  const char **res, const gunichar **consumed, GQuark *quark,
 		  unsigned char **original, gssize *original_length,
 		  struct _vte_table_arginfo_head *params)
 {
@@ -407,6 +425,7 @@ _vte_table_matchi(struct _vte_table *table,
 		*original = table->original;
 		*original_length = table->original_length;
 		*res = table->result;
+		*quark = table->resultq;
 		return table->result;
 	}
 
@@ -434,7 +453,7 @@ _vte_table_matchi(struct _vte_table *table,
 		arginfo->length = i;
 		/* Continue. */
 		return _vte_table_matchi(subtable, candidate + i, length - i,
-					 res, consumed,
+					 res, consumed, quark,
 					 original, original_length, params);
 	}
 
@@ -459,7 +478,7 @@ _vte_table_matchi(struct _vte_table *table,
 		/* Try and continue. */
 		local_result = _vte_table_matchi(subtable,
 					 candidate + i, length - i,
-					 res, consumed,
+					 res, consumed, quark,
 					 original, original_length,
 					 params);
 		if (local_result != NULL) {
@@ -487,7 +506,7 @@ _vte_table_matchi(struct _vte_table *table,
 		arginfo->length = i;
 		/* Continue. */
 		return _vte_table_matchi(subtable, candidate + i, length - i,
-					 res, consumed,
+					 res, consumed, quark,
 					 original, original_length, params);
 	}
 
@@ -502,7 +521,7 @@ _vte_table_matchi(struct _vte_table *table,
 		arginfo->length = 1;
 		/* Continue. */
 		return _vte_table_matchi(subtable, candidate + 1, length - 1,
-					 res, consumed,
+					 res, consumed, quark,
 					 original, original_length, params);
 	}
 
@@ -514,7 +533,7 @@ _vte_table_matchi(struct _vte_table *table,
 
 static void
 _vte_table_extract_numbers(GValueArray **array,
-			   struct _vte_table_arginfo *arginfo)
+			   struct _vte_table_arginfo *arginfo, long increment)
 {
 	GValue value = {0,};
 	GValue subvalue = {0,};
@@ -578,20 +597,38 @@ _vte_table_extract_string(GValueArray **array,
 	g_value_unset(&value);
 }
 
+static void
+_vte_table_extract_char(GValueArray **array,
+			struct _vte_table_arginfo *arginfo, long increment)
+{
+	GValue value = {0,};
+
+	g_value_init(&value, G_TYPE_LONG);
+	g_value_set_long(&value, *(arginfo->start) - increment);
+
+	if (G_UNLIKELY (*array == NULL)) {
+		*array = g_value_array_new(1);
+	}
+	g_value_array_append(*array, &value);
+	g_value_unset(&value);
+}
+
 /* Check if a string matches something in the tree. */
 const char *
 _vte_table_match(struct _vte_table *table,
 		 const gunichar *candidate, gssize length,
 		 const char **res, const gunichar **consumed,
-		 GValueArray **array)
+		 GQuark *quark, GValueArray **array)
 {
 	struct _vte_table *head;
 	const gunichar *dummy_consumed;
 	const char *dummy_res;
+	GQuark dummy_quark;
 	GValueArray *dummy_array;
 	const char *ret;
 	unsigned char *original, *p;
 	gssize original_length;
+	long increment = 0;
 	int i;
 	struct _vte_table_arginfo_head params;
 	struct _vte_table_arginfo *arginfo;
@@ -605,6 +642,10 @@ _vte_table_match(struct _vte_table *table,
 		consumed = &dummy_consumed;
 	}
 	*consumed = candidate;
+	if (G_UNLIKELY (quark == NULL)) {
+		quark = &dummy_quark;
+	}
+	*quark = 0;
 	if (G_UNLIKELY (array == NULL)) {
 		dummy_array = NULL;
 		array = &dummy_array;
@@ -643,6 +684,7 @@ _vte_table_match(struct _vte_table *table,
 		/* Got a literal match. */
 		*consumed = candidate + i;
 		*res = head->result;
+		*quark = head->resultq;
 		return *res;
 	}
 
@@ -650,7 +692,7 @@ _vte_table_match(struct _vte_table *table,
 
 	/* Check for a pattern match. */
 	ret = _vte_table_matchi(table, candidate, length,
-				res, consumed,
+				res, consumed, quark,
 				&original, &original_length,
 				&params);
 	*res = ret;
@@ -663,14 +705,25 @@ _vte_table_match(struct _vte_table *table,
 		do {
 			/* All of the interesting arguments begin with '%'. */
 			if (p[0] == '%') {
+				/* Handle an increment. */
+				if (p[1] == 'i') {
+					increment++;
+					p += 2;
+					continue;
+				}
 				/* Handle an escaped '%'. */
-				if (p[1] == '%') {
+				else if (p[1] == '%') {
 					p++;
 				}
 				/* Handle numeric parameters. */
-				else if ((p[1] == 'd') || (p[1] == 'm')) {
+				else if ((p[1] == 'd') ||
+				    (p[1] == '2') ||
+				    (p[1] == '3') ||
+				    (p[1] == 'm') ||
+				    (p[1] == 'M')) {
 					_vte_table_extract_numbers(array,
-								   arginfo);
+								   arginfo,
+								   increment);
 					p++;
 				}
 				/* Handle string parameters. */
@@ -678,9 +731,16 @@ _vte_table_match(struct _vte_table *table,
 					_vte_table_extract_string(array,
 								  arginfo);
 					p++;
+				}
+				/* Handle a parameter character. */
+				else if (p[1] == '+') {
+					_vte_table_extract_char(array,
+								arginfo,
+								p[2]);
+					p += 2;
 				} else {
 					_vte_debug_print (VTE_DEBUG_PARSE,
-                                                          "Invalid sequence %s\n",
+							  "Invalid termcap sequence %s\n",
 							  original);
 				}
 			} /* else Literal. */
@@ -704,8 +764,8 @@ _vte_table_printi(struct _vte_table *table, const char *lead, int *count)
 
 	/* Result? */
 	if (table->result != NULL) {
-		g_printerr("%s = `%s'\n", lead,
-			table->result);
+		g_printerr("%s = `%s'(%d)\n", lead,
+			table->result, table->increment);
 	}
 
 	/* Literal? */
@@ -856,18 +916,20 @@ main(int argc, char **argv)
 	const gunichar *consumed;
 	char *tmp;
 	gunichar *candidate;
+	GQuark quark;
 	GValueArray *array;
 	g_type_init();
 	table = _vte_table_new();
-	_vte_table_add(table, "ABCDEFG", 7, "ABCDEFG");
-	_vte_table_add(table, "ABCD", 4, "ABCD");
-	_vte_table_add(table, "ABCDEFH", 7, "ABCDEFH");
-	_vte_table_add(table, "ACDEFH", 6, "ACDEFH");
-	_vte_table_add(table, "ACDEF%sJ", 8, "ACDEF%sJ");
-	_vte_table_add(table, "[%mh", 5, "move-cursor");
-	_vte_table_add(table, "[%mm", 5, "character-attributes");
-	_vte_table_add(table, "]3;%s", 7, "set-icon-title");
-	_vte_table_add(table, "]4;%s", 7, "set-window-title");
+	_vte_table_add(table, "ABCDEFG", 7, "ABCDEFG", 0);
+	_vte_table_add(table, "ABCD", 4, "ABCD", 0);
+	_vte_table_add(table, "ABCDEFH", 7, "ABCDEFH", 0);
+	_vte_table_add(table, "ACDEFH", 6, "ACDEFH", 0);
+	_vte_table_add(table, "ACDEF%sJ", 8, "ACDEF%sJ", 0);
+	_vte_table_add(table, "ACDEF%i%mJ", 10, "ACDEF%dJ", 0);
+	_vte_table_add(table, "[%mh", 5, "move-cursor", 0);
+	_vte_table_add(table, "[%mm", 5, "character-attributes", 0);
+	_vte_table_add(table, "]3;%s", 7, "set-icon-title", 0);
+	_vte_table_add(table, "]4;%s", 7, "set-window-title", 0);
 	printf("Table contents:\n");
 	_vte_table_print(table);
 	printf("\nTable matches:\n");
@@ -876,7 +938,7 @@ main(int argc, char **argv)
 		candidate = make_wide(p);
 		array = NULL;
 		_vte_table_match(table, candidate, strlen(p),
-				 &result, &consumed, &array);
+				 &result, &consumed, &quark, &array);
 		tmp = escape(p);
 		printf("`%s' => `%s'", tmp, (result ? result : "(NULL)"));
 		g_free(tmp);
